#!/usr/local/bin/bltwish
namespace import ::blt::*

set Graph(dataName) stock.data

set Graph(colors) {blue pink green orange yellow black gray}
set Graph(current) 0

proc normalize {val} {

    # Only do fraction to decimal conversion if a fraction exists.

    if {[regexp {([0-9]* +)*([0-9]*)/([0-9]*)} $val m whole num denom]} {
        set val [expr $whole + ($num / $denom.0)]
    }
        
    # Delete any commas that might be in the value
    regsub -all "," $val "" val
    
    return $val
}


proc readData {infl} {
    global Data
    
    while {[set len [gets $infl line]] >= 0} {
        if {[string first "N/A" $line] > 0} {continue}
        set id [lindex $line 1]
        foreach n {date id price change pct dt o high low vol} v $line {
            lappend Data($id.$n) [normalize $v]
        }
    }
}

proc fmt {graph sec} {
   return [clock format $sec -format {%m/%d}]
}

proc makeElement {graph name} {
  global Data Graph

  # Create a line showing the stock price when the robot ran.

  set el [$graph  element create "$name Price" -xdata $Data($name.date) \
        -ydata $Data($name.price) -symbol none \
        -color [lindex $Graph(colors) $Graph(current)]]

  incr Graph(current)

  if {$Graph(current) >= [llength $Graph(colors)]} {
      set Graph(current) 0
  }

  $graph element bind $el <Button-3> [list $graph element delete $el]
  $graph legend bind $el <Button-3> [list $graph element delete $el]

  return $graph 
}

proc makeGraph {parent} {
  global Data
    
      # Create the graph

      ::blt::graph $parent.g  -title "Stock Data" -width 600 -height 400 
      $parent.g axis configure x -max [clock seconds]
      
      # Format the x-axis tick labels as Month/Day 
      $parent.g  axis configure x -command fmt

  return $parent.g 
}

set infl [open $Graph(dataName) r]
readData $infl
close $infl

set b [frame .buttons]
set w [frame .graphs]

grid $b -row 0 -column 0
grid $w -row 1 -column 0  -columnspan 20
set graph [makeGraph $w ]
grid $graph

set mb [menubutton $b.m_file -text Files -menu $b.m_file.menu -relief raised]
set menu [menu $b.m_file.menu]
pack $mb -side left -padx 5


foreach range [list "A-E" "F-L" "M-R" "S-Z"] {
    set mb [menubutton $b.m_$range -text $range \
        -menu $b.m_$range.menu -relief raised]
    menu $b.m_$range.menu
    pack $mb -side left -padx 5
}   

foreach id [lsort [array names Data *.date]] {
  
#  # extract the stock symbol from the id - id resembles SUNW.date
  set name [lindex [split $id .] 0]
  
  # Look at the menu buttons, and figure out which range this is in.
  foreach ch [pack slaves $b] {
      foreach {start end} [split [lindex [split $ch _] end] -] {}
       if {([string compare $name $start] >= 0) &&
           ([string compare $name $end ] <= 0)} {
          $ch.menu add command -label $name -command "makeElement $graph $name"
      }
  }
}   

proc Exit {g} {exit}

proc Clear {g} {
    foreach e [$g element names] {
       $g element delete $e
    }
}

proc Postscript {g} {
   set outputfile [tk_getSaveFile]
   if {![string match $outputfile ""]} {
       $g postscript output $outputfile
   }
}

foreach selection {Clear Postscript Exit } {
    $menu add command -label $selection -command "$selection $graph"
}
